package com.tutego.exercise.oop.term;

import java.util.Objects;

//tag::solution[]
public class Term {

  public static void main( String[] args ) {

    BinaryExpression aSquare = new BinaryExpression(
        new Literal( "a" ),
        BinaryExpression.BinaryOperator.POWER,
        new Literal( 2 )
    );

    BinaryExpression bSquare = new BinaryExpression(
        new Literal( "b" ),
        BinaryExpression.BinaryOperator.POWER,
        new Literal( 2 )
    );

    Expression aSquare_plus_bSquare = new BinaryExpression(
        aSquare, BinaryExpression.BinaryOperator.PLUS, bSquare );

    Expression sqrt = new Sqrt( aSquare_plus_bSquare );
    System.out.println( sqrt );
  }
}

interface Expression {

}

class Literal implements Expression {
  public final String variable;

  public Literal( String variable ) {
    this.variable = Objects.requireNonNull( variable );
  }

  public Literal( long value ) {
    this.variable = String.valueOf( value );
  }

  @Override
  public String toString() {
    return variable;
  }
}

class UnaryExpression implements Expression {

  public enum UnaryOperator {PLUS, MINUS}

  public final UnaryOperator unaryOperator;
  public final Expression expression;

  public UnaryExpression( UnaryOperator unaryOperator, Expression expression ) {
    this.unaryOperator = Objects.requireNonNull( unaryOperator );
    this.expression = Objects.requireNonNull( expression );
  }

  @Override public String toString() {
    return switch ( unaryOperator ) {
      case PLUS -> "+";
      case MINUS -> "-";
    } + expression;
  }
}

class BinaryExpression implements Expression {

  public enum BinaryOperator {
    PLUS, MINUS, MULTIPLY, DIVISION, POWER, PLUS_MINUS, MINUS_PLUS, EQUAL, NOT_EQUAL;
  }

  public final Expression left;
  public final BinaryOperator binaryOperator;
  public final Expression right;

  public BinaryExpression( Expression left, BinaryOperator binaryOperator, Expression right ) {
    this.left = Objects.requireNonNull( left );
    this.binaryOperator = Objects.requireNonNull( binaryOperator );
    this.right = Objects.requireNonNull( right );
  }

  @Override public String toString() {
    return left + " " +
           switch ( binaryOperator ) {
             case PLUS -> "+";
             case MINUS -> "-";
             case MULTIPLY -> "*";
             case DIVISION -> "/";
             case POWER -> "^";
             case PLUS_MINUS -> "±";
             case MINUS_PLUS -> "∓";
             case EQUAL -> "=";
             case NOT_EQUAL -> "≠";
           } + " " +
           right;
  }
}

class Parentheses implements Expression {

  public final Expression expression;

  public Parentheses( Expression expression ) {
    this.expression = Objects.requireNonNull( expression );
  }

  @Override
  public String toString() {
    return "(" + expression + ")";
  }
}

class Sqrt implements Expression {

  public final Expression index;
  public final Expression radicand;

  public Sqrt( Expression index, Expression radicand ) {
    this.index = Objects.requireNonNull( index );
    this.radicand = Objects.requireNonNull( radicand );
  }

  public Sqrt( Expression radicand ) {
    this.radicand = Objects.requireNonNull( radicand );
    this.index = null;
  }

  @Override
  public String toString() {
    return ((index != null) ? index : "") + "√( " + radicand + ")";
  }
}
//end::solution[]
